@@ -31,10 +31,9 @@
 
 #include <gkbd-indicator-plugin-manager.h>
 
-#include <gkbd-config-registry-client.h>
-
 typedef struct _gki_globals {
 	XklEngine *engine;
+	XklConfigRegistry *registry;
 
 	GkbdDesktopConfig cfg;
 	GkbdIndicatorConfig ind_cfg;
@@ -88,32 +87,16 @@ gkbd_indicator_set_tooltips (GkbdIndicator * gki, const char *str);
 void
 gkbd_indicator_set_tooltips (GkbdIndicator * gki, const char *str)
 {
-	GtkTooltips *tooltips;
+        g_assert (str == NULL || g_utf8_validate (str, -1, NULL));
 
-	if (str == NULL)
-		return;
-	tooltips = gtk_tooltips_new ();
-	g_object_ref (G_OBJECT (tooltips));
-	gtk_object_sink (GTK_OBJECT (tooltips));
-	g_object_set_data_full (G_OBJECT (gki), "tooltips",
-				tooltips, (GDestroyNotify) g_object_unref);
-	gtk_tooltips_set_tip (tooltips, GTK_WIDGET (gki), str, NULL);
+        gtk_widget_set_tooltip_text (GTK_WIDGET (gki), str);
 
 	if (gki->priv->set_parent_tooltips) {
-		GtkWidget *parent =
-		    gtk_widget_get_parent (GTK_WIDGET (gki));
-		if (parent != NULL) {
-			gtk_tooltips_set_tip (tooltips,
-					      GTK_WIDGET (parent), str,
-					      NULL);
-			g_object_ref (G_OBJECT (tooltips));
-			g_object_set_data_full (G_OBJECT (parent),
-						"gnome-kbd-indicator.tooltips",
-						tooltips, (GDestroyNotify)
-						g_object_unref);
-		}
-	}
-	gtk_tooltips_enable (tooltips);
+		GtkWidget *parent = gtk_widget_get_parent (GTK_WIDGET (gki));
+                if (parent) {
+                        gtk_widget_set_tooltip_text (parent, str);
+                }
+        }
 }
 
 void
@@ -136,7 +119,12 @@ gkbd_indicator_fill (GkbdIndicator * gki)
 	GtkNotebook *notebook = GTK_NOTEBOOK (gki);
 
 	for (grp = 0; grp < total_groups; grp++) {
-		GtkWidget *page, *decorated_page;
+		GtkWidget *page, *decorated_page = NULL;
+		gchar *full_group_name =
+		    (grp <
+		     g_strv_length (globals.
+				    full_group_names)) ?
+		    globals.full_group_names[grp] : "?";
 		page = gkbd_indicator_prepare_drawing (gki, grp);
 
 		if (page == NULL)
@@ -145,7 +133,7 @@ gkbd_indicator_fill (GkbdIndicator * gki)
 		decorated_page =
 		    gkbd_indicator_plugin_manager_decorate_widget
 		    (&globals.plugin_manager, page, grp,
-		     globals.full_group_names[grp], &globals.kbd_cfg);
+		     full_group_name, &globals.kbd_cfg);
 
 		page = decorated_page == NULL ? page : decorated_page;
 
@@ -258,39 +246,46 @@ gkbd_indicator_prepare_drawing (GkbdIndicator * gki, int group)
 			    g_hash_table_new_full (g_str_hash, g_str_equal,
 						   g_free, NULL);
 
-		if (xkl_engine_get_features (globals.engine) &
-		    XKLF_MULTIPLE_LAYOUTS_SUPPORTED) {
-			char *full_layout_name =
-			    (char *) g_slist_nth_data (globals.kbd_cfg.
-						       layouts_variants,
-						       group);
-			char *variant_name;
-			if (!gkbd_keyboard_config_split_items
-			    (full_layout_name, &layout_name,
-			     &variant_name))
-				/* just in case */
-				layout_name = full_layout_name;
 
-			/* make it freeable */
-			layout_name = g_strdup (layout_name);
+		if (group < g_strv_length (globals.short_group_names)) {
+			if (xkl_engine_get_features (globals.engine) &
+			    XKLF_MULTIPLE_LAYOUTS_SUPPORTED) {
+				char *full_layout_name = (char *)
+				    g_slist_nth_data (globals.kbd_cfg.
+						      layouts_variants,
+						      group);
+				char *variant_name;
+				if (!gkbd_keyboard_config_split_items
+				    (full_layout_name, &layout_name,
+				     &variant_name))
+					/* just in case */
+					layout_name = full_layout_name;
 
-			if (globals.short_group_names != NULL) {
-				char *short_group_name =
-				    globals.short_group_names[group];
-				if (short_group_name != NULL
-				    && *short_group_name != '\0') {
-					/* drop the long name */
-					g_free (layout_name);
-					layout_name =
-					    g_strdup (short_group_name);
+				/* make it freeable */
+				layout_name = g_strdup (layout_name);
+
+				if (globals.short_group_names != NULL) {
+					char *short_group_name =
+					    globals.short_group_names
+					    [group];
+					if (short_group_name != NULL
+					    && *short_group_name != '\0') {
+						/* drop the long name */
+						g_free (layout_name);
+						layout_name =
+						    g_strdup
+						    (short_group_name);
+					}
 				}
+			} else {
+				layout_name =
+				    g_strdup (globals.full_group_names
+					      [group]);
 			}
-		} else
-			layout_name =
-			    g_strdup (globals.full_group_names[group]);
+		}
 
 		if (layout_name == NULL)
-			layout_name = g_strdup ("?");
+			layout_name = g_strdup ("??");
 
 		/* Process layouts with repeating description */
 		ppln = &prev_layout_name;
@@ -348,7 +343,8 @@ gkbd_indicator_update_tooltips (GkbdIndicator * gki)
 {
 	XklState *state = xkl_engine_get_current_state (globals.engine);
 	gchar *buf;
-	if (state == NULL || state->group < 0)
+	if (state == NULL || state->group < 0
+	    || state->group >= g_strv_length (globals.full_group_names))
 		return;
 
 	buf = g_strdup_printf (globals.tooltips_format,
@@ -402,12 +398,9 @@ gkbd_indicator_ind_cfg_changed (GConfClient * client,
 					     &globals.kbd_cfg);
 	gkbd_indicator_config_activate (&globals.ind_cfg);
 
-	gkbd_indicator_plugin_manager_toggle_plugins (&globals.
-						      plugin_manager,
-						      &globals.
-						      plugin_container,
-						      globals.ind_cfg.
-						      enabled_plugins);
+	gkbd_indicator_plugin_manager_toggle_plugins
+	    (&globals.plugin_manager, &globals.plugin_container,
+	     globals.ind_cfg.enabled_plugins);
 
 	ForAllIndicators () {
 		gkbd_indicator_reinit_ui (gki);
@@ -418,8 +411,8 @@ static void
 gkbd_indicator_load_group_names (const gchar ** layout_ids,
 				 const gchar ** variant_ids)
 {
-	if (!gkbd_desktop_config_load_remote_group_descriptions_utf8
-	    (&globals.cfg, layout_ids, variant_ids,
+	if (!gkbd_desktop_config_load_group_descriptions
+	    (&globals.cfg, globals.registry, layout_ids, variant_ids,
 	     &globals.short_group_names, &globals.full_group_names)) {
 		/* We just populate no short names (remain NULL) - 
 		 * full names are going to be used anyway */
@@ -431,7 +424,7 @@ gkbd_indicator_load_group_names (const gchar ** layout_ids,
 		if (xkl_engine_get_features (globals.engine) &
 		    XKLF_MULTIPLE_LAYOUTS_SUPPORTED) {
 			GSList *lst = globals.kbd_cfg.layouts_variants;
-			for (i = 0; lst; lst = lst->next,i++) {
+			for (i = 0; lst; lst = lst->next, i++) {
 				globals.full_group_names[i] =
 				    g_strdup ((char *) lst->data);
 			}
@@ -466,8 +459,8 @@ gkbd_indicator_kbd_cfg_callback (GkbdIndicator * gki)
 	}
 
 	gkbd_indicator_load_group_names ((const gchar **) xklrec->layouts,
-					 (const gchar **) xklrec->
-					 variants);
+					 (const gchar **)
+					 xklrec->variants);
 
 	ForAllIndicators () {
 		gkbd_indicator_reinit_ui (gki);
@@ -619,7 +612,7 @@ gkbd_indicator_init (GkbdIndicator * gki)
 		return;
 	}
 
-	gkbd_indicator_set_tooltips (gki, "");
+	gkbd_indicator_set_tooltips (gki, NULL);
 
 	gkbd_indicator_fill (gki);
 	gkbd_indicator_set_current_page (gki);
@@ -665,8 +658,8 @@ gkbd_indicator_global_term (void)
 	gkbd_desktop_config_stop_listen (&globals.cfg);
 	gkbd_indicator_config_stop_listen (&globals.ind_cfg);
 
-	gkbd_indicator_plugin_manager_term_initialized_plugins (&globals.
-								plugin_manager);
+	gkbd_indicator_plugin_manager_term_initialized_plugins
+	    (&globals.plugin_manager);
 	gkbd_indicator_plugin_manager_term (&globals.plugin_manager);
 
 	gkbd_indicator_config_term (&globals.ind_cfg);
@@ -675,6 +668,8 @@ gkbd_indicator_global_term (void)
 
 	gkbd_indicator_plugin_container_term (&globals.plugin_container);
 
+	g_object_unref (G_OBJECT (globals.registry));
+	globals.registry = NULL;
 	g_object_unref (G_OBJECT (globals.engine));
 	globals.engine = NULL;
 	xkl_debug (100, "*** Terminated globals *** \n");
@@ -719,6 +714,11 @@ gkbd_indicator_global_init (void)
 		return;
 	}
 
+	globals.registry =
+	    xkl_config_registry_get_instance (globals.engine);
+
+	xkl_config_registry_load (globals.registry);
+
 	gconf_client = gconf_client_get_default ();
 
 	g_signal_connect (globals.engine, "X-state-changed",
@@ -750,18 +750,14 @@ gkbd_indicator_global_init (void)
 	gkbd_indicator_config_activate (&globals.ind_cfg);
 
 	gkbd_indicator_load_group_names ((const gchar **) xklrec->layouts,
-					 (const gchar **) xklrec->
-					 variants);
+					 (const gchar **)
+					 xklrec->variants);
 	g_object_unref (G_OBJECT (xklrec));
 
 	gkbd_indicator_plugin_manager_init (&globals.plugin_manager);
-	gkbd_indicator_plugin_manager_init_enabled_plugins (&globals.
-							    plugin_manager,
-							    &globals.
-							    plugin_container,
-							    globals.
-							    ind_cfg.
-							    enabled_plugins);
+	gkbd_indicator_plugin_manager_init_enabled_plugins
+	    (&globals.plugin_manager, &globals.plugin_container,
+	     globals.ind_cfg.enabled_plugins);
 	gkbd_desktop_config_start_listen (&globals.cfg,
 					  (GConfClientNotifyFunc)
 					  gkbd_indicator_cfg_changed,
@@ -815,10 +811,9 @@ gkbd_indicator_get_image_filename (guint group)
 {
 	if (!globals.ind_cfg.show_flags)
 		return NULL;
-	return gkbd_indicator_config_get_images_file (&globals.
-						      ind_cfg,
-						      &globals.
-						      kbd_cfg, group);
+	return gkbd_indicator_config_get_images_file (&globals.ind_cfg,
+						      &globals.kbd_cfg,
+						      group);
 }
 
 gdouble
